import QtQuick 2.5
import Fly.Components 1.0

Canvas {
    id: canvas

    property var background: FlySettingsCheckBoxItemData { name: qsTr("Show background"); clickFunc: repaint }
    property var border: FlySettingsCheckBoxItemData { name: qsTr("Show border"); clickFunc: repaint }
    property var marks: FlySettingsCheckBoxItemData { name: qsTr("Show marks"); clickFunc: repaint }
    property var numbers: FlySettingsCheckBoxItemData { name: qsTr("Show numbers"); clickFunc: repaint }
    property var roman: FlySettingsCheckBoxItemData { name: qsTr("Show roman numbers");
        disableDescription: qsTr("Numbers are not displayed");
        enabled: numbers.value; clickFunc: repaint }
    property var day: FlySettingsCheckBoxItemData { name: qsTr("Show current day of month"); clickFunc: repaint }
    property var dark: FlySettingsColorItemData { name: qsTr("Dark color"); clickFunc: repaint }
    property var light: FlySettingsColorItemData { name: qsTr("Light color"); clickFunc: repaint }
    property var red: FlySettingsColorItemData { name: qsTr("Arrow color"); clickFunc: repaint }

    readonly property var romanNumbers: ["I", "II", "III", "IV", "V", "VI", "VII", "VIII", "IX", "X", "XI", "XII"]

    readonly property color blackColor: dark.value
    readonly property color whiteColor: light.value
    readonly property color backColor: Qt.darker(whiteColor, 1.3)
    readonly property color arrowSecColor: red.value
    readonly property color darkBorderColor: root.getColorSumString(blackColor, "#767676")
    readonly property color dayStrokeColor: root.getColorString(0x51, whiteColor)
    readonly property color dayTextColor: root.getColorSumString(blackColor, "#515151")
    readonly property color dayShadowColor: root.getColorString(0x80, blackColor)


    function showSettings() {
        root.addPage(Qt.resolvedUrl("Settings.qml"), { "title": qsTr("Settings"), "reloadMaster": loadSettings, "settingsGroup": settingsGroup })
    }

    function saveSettings() {
        settings.setBool("background", background.value, settingsGroup)
        settings.setBool("border", border.value, settingsGroup)
        settings.setBool("marks", marks.value, settingsGroup)
        settings.setBool("numbers", numbers.value, settingsGroup)
        settings.setBool("roman", roman.value, settingsGroup)
        settings.setBool("day", day.value, settingsGroup)
        settings.setString("dark", root.getColorString(dark.value), settingsGroup)
        settings.setString("light", root.getColorString(light.value), settingsGroup)
        settings.setString("red", root.getColorString(red.value), settingsGroup)
    }

    function loadSettings() {
        background.value = settings.getBool("background", settingsGroup, true)
        border.value = settings.getBool("border", settingsGroup, true)
        marks.value = settings.getBool("marks", settingsGroup, true)
        numbers.value = settings.getBool("numbers", settingsGroup, true)
        roman.value = settings.getBool("roman", settingsGroup, false)
        day.value = settings.getBool("day", settingsGroup, true)
        dark.value = settings.getString("dark", settingsGroup, "black")
        light.value = settings.getString("light", settingsGroup, "white")
        red.value = settings.getString("red", settingsGroup, "red")

        repaint()
    }

    function repaint() {
        requestPaint()
        repaintTimer.restart()
    }

    Timer {
        id: repaintTimer
        interval: 1000
        onTriggered: canvas.repaint()
    }

    onPaint: {
        var ctx = getContext("2d")
        var now = new Date();
        var radius = Math.min(width, height) / 2 * 0.9;
        var hw = width / 2
        var hh = height / 2

        ctx.clearRect(0, 0, width, height)
        drawClock();

        function drawClock() {
            if(background.value || border.value) drawFace();
            if(numbers.value) drawNumbers();
            if(marks.value) drawMarks();
            if(day.value) drawDay();
            drawTime();

            ctx.beginPath();
            ctx.arc(hw, hh, radius * 0.02, 0, Math.PI * 2);
            ctx.fillStyle = arrowSecColor
            ctx.fill();
        }

        function drawFace() {
            ctx.save()
            ctx.beginPath();
            ctx.arc(hw, hh, radius, 0 , Math.PI * 2);
            var grad = ctx.createRadialGradient(hw, 0, radius * 0.5, hw, hh, radius * 0.6);
            grad.addColorStop(0, whiteColor);
            grad.addColorStop(1, backColor);

            if(background.value) {
                ctx.fillStyle = grad;
                ctx.fill();
            }

            grad = ctx.createRadialGradient(hw, hh, 0, hw, hh, radius * 1.05);
            grad.addColorStop(0.91, darkBorderColor);
            grad.addColorStop(0.93, whiteColor);
            grad.addColorStop(0.97, whiteColor);
            grad.addColorStop(1, darkBorderColor);

            if(border.value) {
                ctx.strokeStyle = grad;
                ctx.lineWidth = radius * 0.1;
                ctx.stroke();
            }
            ctx.restore()
        }

        function drawNumbers() {
            ctx.font = radius * 0.15 + "px " + defaultFont;
            ctx.fillStyle = blackColor;
            ctx.textAlign = 'center';
            ctx.textBaseline = 'middle';

            for (var n = 1; n <= 12; n++) {
                if(n === 3 && day.value)
                    continue;

                var theta = (n - 3) * (Math.PI * 2) / 12;
                var x = hw + radius * (marks.value ? 0.78 : 0.83) * Math.cos(theta);
                var y = hh + radius * (marks.value ? 0.78 : 0.83) * Math.sin(theta);
                ctx.fillText(roman.value ? romanNumbers[n - 1] : n, x, y);
            }
        }

        function drawMarks() {
            ctx.strokeStyle = blackColor;

            for(var i = 0; i < 60; i++) {
                var theta = i * Math.PI / 30;
                ctx.lineWidth = i % 5 ? 1 : 3;
                ctx.beginPath();
                ctx.moveTo(hw + radius * 0.95 * Math.cos(theta), hh + radius * 0.95 * Math.sin(theta));
                ctx.lineTo(hw + radius * 0.9 * Math.cos(theta), hh + radius * 0.9 * Math.sin(theta));
                ctx.stroke();
            }
        }

        function drawDay() {
            ctx.save()
            var dayRect = Qt.rect(hw + radius * 0.65, hh - radius * 0.075, radius * 0.18, radius * 0.15)
            ctx.translate(dayRect.x, dayRect.y);

            ctx.fillStyle = whiteColor;
            if(background.value) ctx.fillRect(0, 0, dayRect.width, dayRect.height);

            ctx.lineWidth = 1
            ctx.strokeStyle = dayStrokeColor;
            if(border.value) ctx.strokeRect(0, 0, dayRect.width, dayRect.height);

            ctx.font = radius * 0.11 + "px " + defaultFont;
            ctx.fillStyle = dayTextColor;
            ctx.textAlign = 'center';
            ctx.textBaseline = 'alphabetic';
            ctx.fillText(now.getDate() < 10 ? '0' + now.getDate() : now.getDate(), dayRect.width / 2, dayRect.height / 2 + radius * 0.0375);

            if(border.value) {
                var depth = dayRect.height * 0.1;
                var shadowColor = dayShadowColor;
                var grd = ctx.createLinearGradient(0, 0, 0, depth);
                grd.addColorStop(0, shadowColor); grd.addColorStop(1, 'transparent');
                ctx.fillStyle = grd;
                ctx.fillRect(0, 0, dayRect.width, depth);

                grd = ctx.createLinearGradient(dayRect.width - depth, 0, dayRect.width, 0);
                grd.addColorStop(0, 'transparent'); grd.addColorStop(1, shadowColor);
                ctx.fillStyle = grd;
                ctx.fillRect(dayRect.width - depth, 0, depth, dayRect.height);

                grd = ctx.createLinearGradient(0, dayRect.height - depth, 0, dayRect.height);
                grd.addColorStop(0, 'transparent'); grd.addColorStop(1, shadowColor);
                ctx.fillStyle = grd;
                ctx.fillRect(0, dayRect.height - depth, dayRect.width, depth);

                grd = ctx.createLinearGradient(0, 0, depth, 0);
                grd.addColorStop(0, shadowColor); grd.addColorStop(1, 'transparent');
                ctx.fillStyle = grd;
                ctx.fillRect(0, 0, depth, dayRect.height);
            }
            ctx.restore()
        }

        function drawTime() {
            var hour = now.getHours();
            var minute = now.getMinutes();
            var second = now.getSeconds();

            hour=hour % 12;
            hour=(hour * Math.PI / 6) + (minute * Math.PI / (6 * 60)) + (second * Math.PI / (360 * 60));
            drawHand(hour, radius * 0.5, radius * 0.05, blackColor);

            minute=(minute * Math.PI / 30) + (second * Math.PI / (30 * 60));
            drawHand(minute, radius * 0.75, radius * 0.0375, blackColor);

            second=(second * Math.PI / 30);
            drawHand(second, radius * 0.85, radius * 0.02, arrowSecColor);
        }

        function drawHand(pos, length, width, style) {
            ctx.save()
            ctx.fillStyle = style;
            ctx.translate(canvas.width / 2, canvas.height / 2);
            ctx.rotate(pos);
            ctx.beginPath();
            ctx.moveTo(Math.floor(-width / 2), length * 0.25);
            ctx.lineTo(Math.floor(width / 2), length * 0.25);
            ctx.lineTo(Math.floor(width / 5), -length);
            ctx.lineTo(Math.floor(-width / 5), -length);
            ctx.fill();
            ctx.restore()
        }
    }

    MouseArea { anchors.fill: parent; onClicked: root.startDetachedProcess("fly-admin-date", []) }
    Text { id: dummyText; visible: false }

    Component.onCompleted: loadSettings()
}
